@@ -1,7 +1,6 @@
 
-__version__ = (0, 4, 0, 'r0')
-__versionstr__ = '0.4.0'
+__version__ = (0, 5, 7, '')
+__versionstr__ = '0.5.7'
 
 from djangosanetesting.cases import *
 from djangosanetesting.testrunner import *
-from djangosanetesting.cases import *
